from functools import partial

import numpy as np
from toolz import curry

__all__ = ['debounce', 'filter_durations', 'offset_events', 'merge_overlap']


def debounce(activate_debounce=None, deactivate_debounce=None):
    """Debounce activation and deactivation of events

    Find an occurrence that is active for time >= activate_debounce and
    activate event. Deactivate event only after an occurrence is found
    that is inactive for time >= to deactivate_debounce. Filter out all events
    that fall outside of these bounds. This function is used to prevent
    short duration occurrences from activating or deactivating longer
    events. See mechanical debounce in mechanical switches and relays
    for a similar concept.

    Args:
        activate_debounce (``float``): Default is ``None``.
            Default value does not apply an activate_debounce. Minimum time
            in seconds an occurrence must be active to activate an event.
            (>= activate_debounce)
        deactivate_debounce (``float``): Default is ``None``.
            Default value does not apply an deactivate_debounce. Maximum time
            in seconds an occurrence must be inactive to deactivate an event.
            (>= deactivate_debounce)

    Returns:
        ``callable``: Partial function

    Examples:
        >>> from trouve import find_events
        >>> from trouve.transformations import debounce
        >>> import numpy as np
        >>> y = np.array([2, 3, 2, 3, 4, 5, 2, 3, 3])
        >>> condition = y > 2
        >>> test_events = find_events(condition, 1)
        >>> debounce = debounce(2, 2)
        >>> example_events = find_events(condition, 1, debounce)
        >>> test_events.as_array()
        array([ 0.,  1.,  0.,  1.,  1.,  1.,  0.,  1.,  1.])
        >>> example_events.as_array()
        array([ 0.,  0.,  0.,  1.,  1.,  1.,  1.,  1.,  1.])

    """
    return partial(_debounce, entry_debounce=activate_debounce, exit_debounce=deactivate_debounce)


def _debounce(events, entry_debounce, exit_debounce):
    """Debounce activate and deactivation of events

    See Also:
        trouve.transfomations.debounce

    Args:
        events(:obj: `collections.namedtuple` of int):
            trouve.transformations.RawEvents(starts, stops)
            starts (np.ndarry of int): Index values for event starts
            stops (np.ndarry of int): Index values for event stops
        period (float): Time in seconds between each data point.
            Requires constant increment data that is uniform across
            all data. (1/Hz = s)
        entry_debounce (float): Time in seconds.
        exit_debounce (float): Time in seconds.

    Returns:
        collections.namedtuple:
            trouve.transformations.RawEvents(starts, stops)
            starts (np.ndarry of int): Index values for event starts
            stops (np.ndarry of int): Index values for event stops

    """
    # convert entry_debounce from time seconds to the number of elements
    if entry_debounce is None:
        _entry_debounce = 0
    elif entry_debounce >= 0:
        _entry_debounce = np.ceil(entry_debounce / events._period)
    else:
        raise TypeError('entry_debounce should be a value >= 0 or None')

    # convert entry_debounce from time seconds to the number of elements
    if exit_debounce is None:
        _exit_debounce = 0
    elif exit_debounce >= 0:
        _exit_debounce = np.ceil(exit_debounce / events._period)
    else:
        raise TypeError('exit_debounce should be a value >= 0 or None')

    # from here on out, all calculations are in number of points, not seconds

    start_mask = np.zeros(events._starts.size)
    stop_mask = np.zeros(events._stops.size)
    event_active = False

    for index in np.arange(events._starts.size):
        # get time of the event
        event_length = events._stops[index] - events._starts[index]

        # get time to the next event
        try:
            reset_length = events._starts[index + 1] - events._stops[index]
        except IndexError:
            reset_length = None

        # determine if event entry conditions are met
        if event_active:
            pass
        elif not event_active and event_length >= _entry_debounce:
            event_active = True
        elif not event_active and event_length < _entry_debounce:
            start_mask[index] = 1
            stop_mask[index] = 1
        else:
            raise ValueError

        # determine whether or not an active event stops being
        # active relative to reset_length
        if not event_active or reset_length is None:
            pass
        elif event_active and reset_length >= _exit_debounce:
            event_active = False
        elif event_active and reset_length < _exit_debounce:
            start_mask[index + 1] = 1
            stop_mask[index] = 1
        else:
            raise ValueError

    events._starts = np.ma.masked_where(start_mask > 0, events._starts).compressed()
    events._stops = np.ma.masked_where(stop_mask > 0, events._stops).compressed()

    return events


def filter_durations(min_duration=None, max_duration=None):
    """Filter out durations based on length of time active

    Filter out events that are < min_duration and > max_duration
    (time in seconds).

    Args:
        min_duration (``float``): Default is ``None``.
            Default value does not apply a min_duration filter.
            Filter out events whose duration in seconds is < min_duration.
        max_duration (``float``): Default is ``None``.
            Default value does not apply a max_duration filter. Filter
            out events whose duration in seconds is > max_duration.

    Returns:
        ``callable``:
            Partial function

    Examples:
        >>> from trouve import find_events
        >>> from trouve.transformations import filter_durations
        >>> y = np.array([2, 3, 2, 3, 4, 5, 2, 3, 3])
        >>> condition = y > 2
        >>> test_events = find_events(condition, 1)
        >>> filter_durations = filter_durations(1.5, 2.5)
        >>> example_events = find_events(condition, 1, filter_durations)
        >>> test_events.as_array()
        array([ 0.,  1.,  0.,  1.,  1.,  1.,  0.,  1.,  1.])
        >>> example_events.as_array()
        array([ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  1.,  1.])

    """
    return partial(_filter_durations, mindur=min_duration, maxdur=max_duration)


def _filter_durations(events, mindur, maxdur):
    """Filter out events based  on duration

    See Also:
        trouve.transfomations.filter_durations

    Args:
        events (:obj: `collections.namedtuple` of int):
            trouve.transformations.RawEvents(starts, stops)
            starts (np.ndarry of int): Index values for event starts
            stops (np.ndarry of int): Index values for event stops
        period (float): Time in seconds between each data point.
            Requires constant increment data that is uniform across
            all data. (1/Hz = s)
        mindur (float): Time in seconds. Any occurrence whose time is
            < mindur is filtered out.
        maxdur (float): Time in seconds. Any occurrence whose time is
            > maxdur is filtered out.

    Returns:
        collections.namedtuple:
            trouve.transformations.RawEvents(starts, stops)
            starts (np.ndarry of int): Index values for event starts
            stops (np.ndarry of int): Index values for event stops

    """
    # convert mindur from time seconds to the number of elements
    if mindur is None:
        _mindur = 0
    elif mindur >= 0:
        _mindur = np.ceil(mindur / events._period)
    else:
        raise TypeError('mindur should be a value >= 0 or None')

    # convert maxdur from time seconds to the number of elements
    if maxdur is None:
        _maxdur = None
    elif maxdur >= 0:
        _maxdur = np.floor(maxdur / events._period)
    else:
        raise TypeError('maxdur should be a value >= 0 or None')

    # from here on out, all calculations are in number of points, not seconds

    event_lengths = events._stops - events._starts
    if _maxdur is not None:
        condition = ((event_lengths < _mindur) |
                     (event_lengths > _maxdur))
    else:
        condition = (event_lengths < _mindur)

    events._starts = np.ma.masked_where(condition, events._starts).compressed()
    events._stops = np.ma.masked_where(condition, events._stops).compressed()

    return events


def offset_events(start_offset=None, stop_offset=None):
    """Apply an offset to event start and stops

    Offset the starts and stops of events by the time in seconds
    specified by start_offset and stop_offset.

    Args:
        start_offset (``float``): Default is ``None``.
            Time in seconds to offset event starts. Value must be <= 0.
        stop_offset (float): Default is ``None``.
            Time in seconds to offset event stops. Value must be >= 0.

    Returns:
        ``callable``:
            Partial function

    Examples:
        >>> from trouve import find_events
        >>> from trouve.transformations import offset_events
        >>> y = np.array([2, 2, 2, 3, 4, 5, 2, 2, 2])
        >>> condition = y > 2
        >>> test_events = find_events(condition, 1)
        >>> offset_events = offset_events(-1, 1)
        >>> example_events = find_events(condition, 1, offset_events)
        >>> test_events.as_array()
        array([ 0.,  0.,  0.,  1.,  1.,  1.,  0.,  0.,  0.])
        >>> example_events.as_array()
        array([ 0.,  0.,  1.,  1.,  1.,  1.,  1.,  0.,  0.])

    """
    return partial(_offset_events, start_offset=start_offset, stop_offset=stop_offset)


def _offset_events(events, start_offset, stop_offset):
    """Apply an offset to event start and stops

    See Also:
        trouve.transfomations.offset_events

    Args:
        events (:obj: `collections.namedtuple` of int):
            trouve.transformations.RawEvents(starts, stops)
            starts (np.ndarry of int): Index values for event starts
            stops (np.ndarry of int): Index values for event stops
        period (float): Time in seconds between each data point.
            Requires constant increment data that is uniform across
            all data. (1/Hz = s)
        condition_size (float):
        start_offset: Time in seconds.
        stop_offset: Time in seconds.

    Returns:
        collections.namedtuple:
            trouve.transformations.RawEvents(starts, stops)
            starts (np.ndarry of int): Index values for event starts
            stops (np.ndarry of int): Index values for event stops

    """
    if start_offset is None:
        starts = events._starts
    elif start_offset <= 0:
        starts = events._starts + np.floor(start_offset/events._period).astype(np.int32)
    else:
        raise ValueError('start_offset must be None or <= 0')

    if stop_offset is None:
        stops = events._stops
    elif stop_offset >= 0:
        stops = events._stops + np.ceil(stop_offset/events._period).astype(np.int32)
    else:
        raise ValueError('stop_offset must be None or >= 0')

    np.clip(starts, 0, events._condition_size, out=events._starts)
    np.clip(stops, 0, events._condition_size, out=events._stops)

    return events


def merge_overlap(events):
    """Merge any events that overlap

    Some events such as offset_events can cause events to overlap. If
    this transformation is applied, any events that overlap will become
    one contiguous event.

    Args:
        events (:any:`Events`):

    Returns:
        :any:`Events`:

    Examples:
        >>> from trouve import find_events
        >>> from trouve.transformations import offset_events, merge_overlap
        >>> y = np.array([2, 3, 2, 3, 4, 5, 2, 2, 2])
        >>> condition = y > 2
        >>> offset_events = offset_events(-1, 1)
        >>> test_events = find_events(condition, 1, offset_events)
        >>> example_events = find_events(condition, 1, offset_events,
        ... merge_overlap)
        >>> test_events.as_array()
        array([ 1.,  1.,  1.,  1.,  1.,  1.,  1.,  0.,  0.])
        >>> example_events.as_array()
        array([ 1.,  1.,  1.,  1.,  1.,  1.,  1.,  0.,  0.])
        >>> len(test_events)
        2
        >>> len(example_events)
        1

    """
    _mask = np.array([False])
    mask = (events._starts[1:] <= events._stops[:-1])
    events._starts = np.ma.masked_where(np.append(_mask, mask),
                                        events._starts).compressed()
    events._stops = np.ma.masked_where(np.append(mask, _mask),
                                       events._stops).compressed()

    return events


def main():
    input_events = RawEvents(np.array([2, 7, 11]), np.array([4, 10, 12]))
    #test_events = _debounce(input_events, period=1, entry_debounce=2, exit_debounce=0)
    test_events = _filter_durations(input_events, period=1, mindur=2, maxdur=3.1)

if __name__ == '__main__':
    main()



import numpy as np
import pandas as pd
from toolz import pipe, curry

from trouve.events import Events


@curry
def find_events(condition, period, *transformations, name='events'):
    """Find events based off a condition

    Find events based off a ``bool`` conditional array and apply a sequence
    of transformation functions to them.

    Args:
        condition (``numpy.ndarray`` or ``pandas.Series`` of ``bool``):
            Boolean conditional array.
        period (``float``):
            Time in seconds between each data point. Requires constant
            increment data that is uniform across the array. (1/Hz = s)
        *transformations (sequence of ``callable`` 's, optional):
            Ordered sequence of transformation functions to apply to
            events. Transformations are applied via ``toolz.pipe()``
        name (``str``, optional): Default is ``'events'``.
            User provided name for events.

    Returns:
        :class:`trouve.events.Events`:
            Returns events found from ``condition`` with any supplied
            ``*transformations`` applied.

    Examples:
        >>> from trouve import find_events
        >>> from trouve.transformations import *
        >>> import numpy as np
        >>> deb = debounce(2, 2)
        >>> offsets = offset_events(-1,2)
        >>> filt_dur = filter_durations(3, 5)
        >>> x = np.array([4, 5, 1, 2, 3, 4, 5, 1, 3])
        >>> condition = (x > 2)
        >>> no_transforms = find_events(condition, 1)
        >>> events = find_events(condition, 1, deb,
        ... filt_dur, offsets, name='example')
        >>> no_transforms.as_array()
        array([ 1.,  1.,  0.,  0.,  1.,  1.,  1.,  0.,  1.])
        >>> events.as_array()
        array([ 0.,  0.,  0.,  1.,  1.,  1.,  1.,  1.,  1.])

    """
    if type(condition) is pd.core.series.Series:
        condition = condition.values

    starts, stops = _apply_condition(condition)
    raw_events = Events(starts, stops, period, name, condition.size)

    transformed_events = pipe(raw_events, *transformations)

    return transformed_events


def _apply_condition(condition):
    """Distill an array of bool into start and stop indexes

    Convert a conditional array of bools into two numpy.ndarrays of
    integers where starts are the indexes where condition goes from
    False to True. Stops are the indexes where condition goes from
    True to False.

    Args:
        condition (numpy.array of bool):

    Returns:
        tuple(numpy.ndarray, numpy.ndarray):

    """
    if isinstance(condition, pd.core.series.Series):
        condition = condition.values

    mask = (condition > 0).view('i1')
    slice_index = np.arange(mask.size + 1, dtype=np.int32)

    # Determine if condition is active at array start, set to_begin accordingly
    if mask[0] == 0:
        to_begin = np.array([0], dtype='i1')
    else:
        to_begin = np.array([1], dtype='i1')

    # Determine if condition is active at array end, set to_end accordingly
    if mask[-1] == 0:
        to_end = np.array([0], dtype='i1')
    else:
        to_end = np.array([-1], dtype='i1')

    deltas = np.ediff1d(mask, to_begin=to_begin, to_end=to_end)

    starts = np.ma.masked_where(deltas < 1, slice_index).compressed()
    stops = np.ma.masked_where(deltas > -1, slice_index).compressed()

    return starts, stops



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from codecs import open
from os import path

from setuptools import find_packages, setup

from trouve import __version__

base = path.abspath(path.dirname(__file__))


def install_requires():
    with open(path.join(base, 'requirements.txt'), encoding='utf-8') as file:
        return file.read().splitlines()


def dev_requires():
    with open(path.join(base, 'dev_requirements.txt'), encoding='utf-8') as file:
        return file.read().splitlines()


def long_description():
    with open(path.join(base, 'README.rst'), encoding='utf-8') as file:
        return file.read()


kwargs = dict(
    name='trouve',
    version=__version__,
    description='Event detection and transformation for time-series data',
    long_description=long_description(),
    author='Ry Whittington',
    author_email='rwhitt2049@gmail.com',
    license='MIT',
    url='https://github.com/rwhitt2049/trouve',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Scientific/Engineering'
    ],
    keywords='time_series, timeseries, iot, sensor',
    packages=find_packages(exclude=['contrib', 'documentation', 'tests*']),
    install_requires=install_requires(),
    package_data={},
    data_files=[],
    entry_points={},
    test_suite='tests',
    tests_require=dev_requires()
)

setup(**kwargs)



from collections import namedtuple

import numpy as np
import pandas as pd


Occurrence = namedtuple('Occurrence', 'start stop slice duration')


class Events(object):
    """Object to represent events found in time series data

    A representation of events based off a ``bool`` conditional array.

    Attributes:
        name (``str``): User provided name for events.
        _starts (``np.array`` of ``int``): The index for event starts
        _stops (``np.array`` of ``int``): The index for event stops
        _period (``float``): Time between each value of the original condition array
        _condition_size (``int``): The size of the original condition array

    """
    def __init__(self, starts, stops, period, name, condition_size):
        self.name = name
        self._starts = starts
        self._stops = stops
        self._period = period
        self._condition_size = condition_size

    @property
    def durations(self):
        """Return a ``numpy.ndarray`` of event durations in seconds.

        Examples:
            >>> from trouve import find_events
            >>> x = np.array([2, 2, 4, 5, 3, 2])
            >>> condition = x == 2
            >>> events = find_events(condition, 1)
            >>> print(events.as_array())
            [ 1.  1.  0.  0.  0.  1.]
            >>> print(events.durations)
            [2 1]

        """
        durations = (self._stops - self._starts) * self._period
        return durations

    # TODO force defaults to np.int8 datatype
    # specify false and true values as None, check if values and dtype is noe
    # and specify appropriately
    def as_array(self, false_values=0, true_values=1, dtype=np.float):
        """Returns a ``numpy.ndarray`` identifying found events

        Useful for plotting.

        Parameters:
            false_values(``float``, optional): Default is 0.
                Value of array where events are not active.
            true_values (``float``, optional): Default is 1.
                Value of array where events are active.
            dtype (``numpy.dtype``, optional): Default is ``numpy.float``.
                Datatype of returned array.

        Returns:
            ``numpy.ndarray``:
                An array where values are coded to identify when events are active
                or inactive.

        Examples:
            >>> from trouve import find_events
            >>> x = np.array([2, 2, 4, 5, 3, 2])
            >>> condition = x > 2
            >>> print(condition)
            [False False  True  True  True False]
            >>> events = find_events(condition, 1)
            >>> print(events.as_array())
            [ 0.  0.  1.  1.  1.  0.]

        """
        output = np.ones(self._condition_size, dtype=dtype) * false_values
        for start, stop in zip(self._starts, self._stops):
            output[start:stop] = 1 * true_values
        return output.astype(dtype)

    def as_series(self, false_values=0, true_values=1, name=None):
        """Returns a ``pandas.Series`` identifying found events

        Useful for plotting and for filtering a ``pandas.DataFrame``

        Parameters:
            false_values(``float``, optional): Default is 0.
                Value of array where events are not active.
            true_values (``float``, optional): Default is 1.
                Value of array where events are active.
            name (``str``, optional): Default is :attr:`Events.name`.
                Name of series.

        Returns:
            ``pandas.Series``:
                A series where values are coded to identify when events are active
                or inactive.

        Examples:
            >>> from trouve import find_events
            >>> x = np.array([2, 2, 4, 5, 3, 2])
            >>> condition = x > 2
            >>> print(condition)
            [False False  True  True  True False]
            >>> events = find_events(condition, 1)
            >>> print(events.as_series())
            0    0.0
            1    0.0
            2    1.0
            3    1.0
            4    1.0
            5    0.0
            Name: events, dtype: float64

        """
        if name is None:
            name = self.name
        data = self.as_array(false_values=false_values, true_values=true_values)
        return pd.Series(data=data, name=name)

    def as_mask(self):
        """Returns a ``numpy.ndarray`` ``bool`` mask for use with ``numpy.ma``

        This method returns a ``np.ndarray`` of ``bool`` where values are
        False where the condition is met, and True where the conditions
        are not met. Trouve treats conditionals opposite of how numpy
        treats them. That is to say that numpy will mask out values in
        an array that meet the condition, however Trouve is by design
        more interested in finding and keeping events that meet the
        given condition. This method makes it more convenient to
        interact with the numpy masked array module as it will return
        a mask that can be directly used by the ``numpy.ma`` module.
        This is similar to what the tilde does for bool arrays.

        Returns:
            ``np.ndarray``:
                An array of bools where the values are ``True`` when the condition
                isn't met and ``False`` when the conditions are met.

        Examples:
            >>> from trouve import find_events
            >>> x = np.array([2, 2, 4, 5, 3, 2])
            >>> condition = x > 2
            >>> print(condition)
            [False False  True  True  True False]
            >>> events = find_events(condition, 1)
            >>> print(events.as_array())
            [ 0.  0.  1.  1.  1.  0.]
            >>> print(events.as_mask())
            [ True  True False False False  True]
            >>> print(np.ma.masked_where(events.as_mask(), x))
            [-- -- 4 5 3 --]

        """
        return self.as_array(1, 0, np.int8).view(bool)

    def __iter__(self):
        self._i = 0
        return self

    def __next__(self):
        """Iterate through ``Events._starts`` and ``Events._stops`` and return an :class:`.Occurrence`

        Examples:
            >>> import numpy as np
            >>> from trouve import find_events
            >>> x = np.array([0, 1, 1, 0, 1, 0])
            >>> example = find_events(x, 1, name='example')
            >>> for event in example:
            ...     print(event)
            ...
            Occurrence(start=1, stop=2, slice=slice(1, 3, None), duration=2)
            Occurrence(start=4, stop=4, slice=slice(4, 5, None), duration=1)

        """
        try:
            occurrence = Occurrence(
                start=self._starts[self._i],
                stop=self._stops[self._i] - 1,
                slice=slice(self._starts[self._i], self._stops[self._i]),
                duration=(self._stops[self._i] - self._starts[self._i]) * self._period
            )
            self._i += 1
            return occurrence
        except IndexError:
            raise StopIteration

    def __getitem__(self, item):
        """Get a specific :class:`.Occurrence`

        Examples:
            >>> import numpy as np
            >>> from trouve import find_events
            >>> x = np.array([0, 1, 1, 0, 1, 0])
            >>> example = find_events(x, 1, name='example')
            >>> first_event = example[0]
            >>> print(first_event)
            Occurrence(start=1, stop=2, slice=slice(1, 3, None), duration=2)

        """
        occurrence = Occurrence(
            start=self._starts[item],
            stop=self._stops[item]-1,
            slice=slice(self._starts[item], self._stops[item]),
            duration=(self._stops[item] - self._starts[item]) * self._period
        )
        return occurrence

    def __len__(self):
        """Returns the number of events found

        Redirects to :any:`Events._starts` and returns ``Events._starts.size``

        Examples:
            >>> import numpy as np
            >>> from trouve import find_events
            >>> x = np.array([0, 1, 1, 0, 1, 0])
            >>> example = find_events(x, 1, name='example')
            >>> len(example)
            2

        """
        return self._starts.size

    def __repr__(self):
        return (
            '{__class__.__name__}(_starts={_starts!r}, '
            '_stops={_stops!r}, '
            '_period={_period!r}, '
            'name={name!r}, '
            '_condition_size={_condition_size!r})'
        ).format(__class__=self.__class__, **self.__dict__)

    def __str__(self):
        """Prints a summary of the events

        Examples:
            >>> import numpy as np
            >>> from trouve import find_events
            >>> x = np.array([0, 1, 1, 0, 1, 0])
            >>> example = find_events(x, 1, name='example')
            >>> print(example)
            example
            Number of events: 2
            Min, Max, Mean Duration: 1.000s, 2.000s, 1.500s

        """
        args = [len(self),
                np.min(self.durations),
                np.max(self.durations),
                np.mean(self.durations)]

        kwargs = {'name': '{}'.format(self.name),
                  'period': '{}s'.format(self._period)}
        return (
            '{name}'
            '\nNumber of events: {0}'
            '\nMin, Max, Mean Duration: {1:.3f}s, {2:.3f}s, {3:.3f}s'
        ).format(*args, **kwargs)

    def __eq__(self, other):
        """Determine if two Events objects are identical

        Compares :attr:`Events._starts`, :attr:`Events._stops`, :attr:`Events._period`
        and :attr:`Events.condition.size` to determine if equality of two events.
        Events objects can have different names and still be equal.

        Examples:
            >>> import numpy as np
            >>> from trouve import find_events
            >>> x = np.array([0, 1, 1, 0, 1, 0])
            >>> example = find_events(x, 1, name='example')
            >>> other = find_events(x, 1, name='other')
            >>> id(example) # doctest: +SKIP
            2587452050568
            >>> id(other) # doctest: +SKIP
            2587452084352
            >>> example == other
            True
            >>> example != other
            False

        """
        if (np.all(self._starts == other._starts)
                and np.all(self._stops == other._stops)
                and self._period == other._period
                and self._condition_size == other._condition_size):
            return True
        else:
            return False

    def __hash__(self):
        """Numpy arrays aren't hashable.

        Researching solution that doesn't require something beyond
        standard lib.
        """
        return id(self)


def main():
    import logging
    from trouve import debounce, offset_events, filter_durations
    logger = logging.getLogger('trouve')
    logger.setLevel(logging.DEBUG)
    np.random.seed(10)
    debounce = debounce(2, 2)
    offset_events = offset_events(-11, 2)
    filter_durations = filter_durations(3, 5)

    x = np.random.random_integers(0, 1, 20)
    y = np.random.random_integers(2, 4, 20)
    condition = (x > 0) & (y <= 3)

    events = find_events(condition, 1, debounce, filter_durations,
                         offset_events, name='example')
    print(repr(events))
    print(events.as_array())
    print(events.durations)
    events.as_array()
    print(events)


if __name__ == '__main__':
    import sys
    sys.exit(main())
